@@ -5,6 +5,7 @@
 #  Distributed under the MIT License.
 #  See LICENSE for more info.
 
+from colour import Color
 from OpenGL.GL import *
 
 from .gldrawable import GLDrawable
@@ -14,24 +15,24 @@ class BackgroundGL(GLDrawable):
     def __init__(self, element=None, *args, **kwargs):
         super().__init__(element, *args, **kwargs)
         self._top_color = [0.1, 0.2, 0.3]
-        self._bot_color = [0.4, 0.5, 0.6]
+        self._bottom_color = [0.4, 0.5, 0.6]
 
     @property
     def top_color(self) -> list:
         return self._top_color
 
     @property
-    def bot_color(self) -> list:
-        return self._bot_color
+    def bottom_color(self) -> list:
+        return self._bottom_color
 
     @top_color.setter
     def top_color(self, color: list) -> None:
         self._top_color = color
         self.notify()
 
-    @bot_color.setter
-    def bot_color(self, color: list) -> None:
-        self._bot_color = color
+    @bottom_color.setter
+    def bottom_color(self, color: list) -> None:
+        self._bottom_color = color
         self.notify()
 
     def __dir__(self) -> list:
