@@ -11,6 +11,7 @@ from qtpy.QtWidgets import QFileDialog
 from qtpy.QtWidgets import QVBoxLayout
 from qtpy.QtWidgets import QWidget
 
+from .camerawidget import CameraWidget
 from .toolbar import ToolBar
 from .treewidget import TreeWidget
 from .iconcollection import IconCollection
@@ -22,30 +23,38 @@ class Container(QWidget):
         QWidget.__init__(self, parent)
         self.setAcceptDrops(True)
 
-        self.toolbar = ToolBar(self)
-        self.tree = TreeWidget()
-        self.viewer = IntegrableViewer(self)
+        # Container
+        self.setWindowTitle('BlastSight (Container)')
+        self.resize(600, 500)
 
+        # Widgets
+        self.toolbar = ToolBar(self)
+        self.viewer = IntegrableViewer(self)
+        self.camera = CameraWidget()
+        self.tree = TreeWidget()
+
+        # Layout
         self.layout = QVBoxLayout(self)
         self.layout.addWidget(self.viewer)
         self.layout.addWidget(self.toolbar)
 
-        # Container
-        self.setWindowTitle('BlastSight (Container)')
-        self.setWindowIcon(IconCollection.get('blastsight.png'))
-        self.resize(600, 500)
-
-        # Tree
-        self.tree.setWindowTitle('Drawables')
-
         # Actions
-        self.toolbar.connect_tree(self.tree)
-        self.toolbar.connect_viewer(self.viewer)
-        self.tree.connect_viewer(self.viewer)
+        self.toolbar.auto_connect(self.tree, self.viewer, self.camera)
 
         actions = self.toolbar.action_collection
         actions.action_take_screenshot.triggered.connect(self.handle_screenshot)
 
+        # Icons
+        try:
+            self.set_widgets_icon(parent.windowIcon())
+        except AttributeError:
+            self.set_widgets_icon(IconCollection.get('blastsight.png'))
+
+    def set_widgets_icon(self, icon) -> None:
+        self.setWindowIcon(icon)
+        self.camera.setWindowIcon(icon)
+        self.tree.setWindowIcon(icon)
+
     def handle_screenshot(self) -> None:
         (path, selected_filter) = QFileDialog.getSaveFileName(
             parent=self.viewer,
