@@ -8,28 +8,28 @@
 import pathlib
 
 from datetime import datetime
+from qtpy.QtCore import QDir
 from qtpy.QtCore import Qt
 from qtpy.QtCore import QFileInfo
 from qtpy.QtCore import QSettings
 from qtpy.QtCore import QThreadPool
 from qtpy.QtWidgets import QFileDialog
 from qtpy.QtWidgets import QMainWindow
-from qtpy.QtWidgets import QMenu
+from qtpy.QtWidgets import QProgressBar
 
-from .cameradialog import CameraDialog
-from .helpdialog import HelpDialog
-from .aboutdialog import AboutDialog
-from .threadworker import ThreadWorker
+from .dialogs.helpdialog import HelpDialog
+from .dialogs.aboutdialog import AboutDialog
 from .iconcollection import IconCollection
 
 from .tools import uic
+from ..threadworker import ThreadWorker
 
 
 class MainWindow(QMainWindow):
     def __init__(self, parent=None):
         QMainWindow.__init__(self, parent)
         uic.loadUi(f'{pathlib.Path(__file__).parent}/UI/mainwindow.ui', self)
-        self.resize(1000, 500)
+        self.resize(1000, 620)
 
         self.setWindowTitle('BlastSight')
         self.setWindowIcon(IconCollection.get('blastsight.png'))
@@ -67,10 +67,15 @@ class MainWindow(QMainWindow):
                     'All Files (*.*)',
         }
 
+        # Progress bar (hidden by default)
+        self.progress_bar = QProgressBar(self.statusBar)
+        self.progress_bar.setValue(0)
+        self.progress_bar.setMaximumWidth(self.width() / 5)
+        self.progress_bar.hide()
+        self.statusBar.addPermanentWidget(self.progress_bar)
+
         # Extra actions
         actions = self.toolbar.action_collection
-        self.toolbar.insertAction(actions.action_plan_view,
-                                  actions.action_camera_properties)
         self.toolbar.addAction(actions.action_quit)
         self.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
 
@@ -81,6 +86,13 @@ class MainWindow(QMainWindow):
         if not actions.action_autofit_to_screen.isChecked():
             actions.action_autofit_to_screen.trigger()
 
+        # Set animated as True when starting the app
+        if not actions.action_animated_viewer.isChecked():
+            actions.action_animated_viewer.trigger()
+
+        # Set camera widget as hidden
+        self.dockWidget_camera.hide()
+
         # self.title = self.windowTitle()
         # self.viewer.signal_fps_updated.connect(lambda x: self.setWindowTitle(f'{self.title} (FPS: {x:.1f})'))
 
@@ -108,6 +120,7 @@ class MainWindow(QMainWindow):
         self.menu_View.addAction(actions.action_fit_to_screen)
         self.menu_View.addSeparator()
         self.menu_View.addAction(actions.action_autofit_to_screen)
+        self.menu_View.addAction(actions.action_animated_viewer)
         self.menu_View.addAction(actions.action_turbo_rendering)
         self.menu_View.addSeparator()
         self.menu_View.addAction(actions.action_perspective_projection)
@@ -127,9 +140,11 @@ class MainWindow(QMainWindow):
     def connect_actions(self) -> None:
         actions = self.toolbar.action_collection
 
-        # Toolbar/Tree
-        self.toolbar.connect_tree(self.dockWidget)
+        # Toolbar/Tree/Camera
         self.toolbar.connect_viewer(self.viewer)
+        self.toolbar.connect_tree(self.dockWidget_tree)
+        self.toolbar.connect_camera(self.dockWidget_camera)
+        self.cameraWidget.connect_viewer(self.viewer)
         self.treeWidget.connect_viewer(self.viewer)
         self.treeWidget.enable_exportability(True)
 
@@ -149,8 +164,7 @@ class MainWindow(QMainWindow):
         actions.action_quit.triggered.connect(self.close)
 
         # View
-        actions.action_camera_properties.triggered.connect(self.handle_camera)
-        actions.action_take_screenshot.triggered.connect(self.handle_screenshot)
+        actions.action_take_screenshot.triggered.connect(self.slot_screenshot)
 
         # Tools
         actions.action_detection_mode.triggered.connect(self.slot_detection_mode)
@@ -167,14 +181,17 @@ class MainWindow(QMainWindow):
         self.viewer.signal_mode_updated.connect(self.slot_mode_updated)
         self.viewer.signal_mesh_clicked.connect(self.slot_mesh_clicked)
         self.viewer.signal_mesh_distances.connect(self.slot_mesh_distances)
-        self.viewer.signal_mesh_sliced.connect(self.slot_mesh_sliced)
-        self.viewer.signal_blocks_sliced.connect(self.slot_blocks_sliced)
+        self.viewer.signal_slice_description.connect(self.slot_slice_description)
 
         self.viewer.signal_load_success.connect(self.slot_element_load_success)
         self.viewer.signal_load_failure.connect(self.slot_element_load_failure)
         self.viewer.signal_export_success.connect(self.slot_element_export_success)
         self.viewer.signal_export_failure.connect(self.slot_element_export_failure)
 
+        self.viewer.signal_process_updated.connect(self.slot_process_updated)
+        self.viewer.signal_process_started.connect(self.slot_process_started)
+        self.viewer.signal_process_finished.connect(self.slot_process_finished)
+
         # TreeWidget actions
         self.treeWidget.signal_export_mesh.connect(self.dialog_export_mesh)
         self.treeWidget.signal_export_blocks.connect(self.dialog_export_blocks)
@@ -199,6 +216,7 @@ class MainWindow(QMainWindow):
     Status bar updates
     """
     def slot_element_load_success(self, _id: int) -> None:
+        self.viewer.update_turbo()
         self.statusBar.showMessage(f'Load successful (id: {_id}).')
 
     def slot_element_load_failure(self) -> None:
@@ -220,13 +238,33 @@ class MainWindow(QMainWindow):
         id_list = [attr.get('id', -1) for attr in mesh_attributes]
         self.statusBar.showMessage(f'Detected meshes: {id_list}')
 
-    def slot_mesh_sliced(self, slice_dict: dict) -> None:
-        slice_list = slice_dict.get('slices', [])
+    """
+    Slots for cross-sections
+    """
+    def slot_slice_description(self, description: dict) -> None:
+        origin = description.get('origin')
+        normal = description.get('normal')
+        up = description.get('up')
 
+        # Slice elements
+        mesh_slices = self.viewer.slice_meshes(origin, normal)
+        block_slices = self.viewer.slice_blocks(origin, normal)
+
+        # Add to elements
+        self.add_mesh_slices(mesh_slices)
+        self.add_block_slices(block_slices)
+
+        # Auto-rotate camera to meet cross-section
+        self.viewer.set_camera_from_vectors(normal, up)
+
+        # Auto-exit slice mode
+        self.viewer.set_normal_mode()
+
+    def add_mesh_slices(self, slice_list: list) -> None:
         for sliced_meshes in slice_list:
             slices = sliced_meshes.get('vertices')
-            origin_id = sliced_meshes.get('origin_id')
-            mesh = self.viewer.get_drawable(origin_id)
+            mesh_id = sliced_meshes.get('element_id')
+            mesh = self.viewer.get_drawable(mesh_id)
 
             if 'SLICE' in str(mesh.name):
                 continue
@@ -235,16 +273,14 @@ class MainWindow(QMainWindow):
                 self.viewer.lines(vertices=vert_slice,
                                   color=mesh.color,
                                   name=f'MESHSLICE_{i}_{mesh.name}',
-                                  extension=mesh.extension,
+                                  extension='csv',
                                   loop=True)
 
-    def slot_blocks_sliced(self, slice_dict: dict) -> None:
-        slice_list = slice_dict.get('slices', [])
-
+    def add_block_slices(self, slice_list: list) -> None:
         for sliced_blocks in slice_list:
             indices = sliced_blocks.get('indices')
-            origin_id = sliced_blocks.get('origin_id')
-            block = self.viewer.get_drawable(origin_id)
+            block_id = sliced_blocks.get('element_id')
+            block = self.viewer.get_drawable(block_id)
 
             if 'SLICE' in str(block.name):
                 continue
@@ -255,74 +291,72 @@ class MainWindow(QMainWindow):
                                vmin=block.vmin,
                                vmax=block.vmax,
                                colormap=block.colormap,
-                               name=f'BLOCKSLICE_{block.name}',
-                               extension=block.extension,
                                block_size=block.block_size,
-                               alpha=1.0,
-                               )
+                               name=f'BLOCKSLICE_{block.name}',
+                               extension='csv')
 
     """
     Common functionality for loading/exporting
     """
-    def _threaded_load(self, method: classmethod, path: str, *args, **kwargs) -> None:
-        self.statusBar.showMessage('Loading...')
-
-        worker = ThreadWorker(method, path, *args, **kwargs)
+    @staticmethod
+    def _thread_runner(method: callable, *args, **kwargs) -> None:
+        worker = ThreadWorker(method, *args, **kwargs)
         QThreadPool.globalInstance().start(worker)
 
-    def _threaded_export(self, method: classmethod, path: str, _id: int) -> None:
-        self.statusBar.showMessage('Exporting...')
-
-        worker = ThreadWorker(method, path, _id)
-        QThreadPool.globalInstance().start(worker)
-
-    def _dialog_load_element(self, method: classmethod, hint: str, *args, **kwargs) -> None:
+    def _dialog_load_element(self, loader: classmethod, hint: str, *args, **kwargs) -> None:
         (paths, selected_filter) = QFileDialog.getOpenFileNames(
             parent=self,
             directory=self.last_dir,
             filter=self.filters_dict.get(hint))
 
-        path_list = [p for p in paths if p != '']
-        for path in sorted(path_list):
-            self._threaded_load(method, path, *args, **kwargs)
-            self.last_dir = QFileInfo(path).absoluteDir().absolutePath()
+        path_list = sorted([p for p in paths if p != ''])
+        if len(path_list) > 0:
+            self.statusBar.showMessage(f'Loading {len(path_list)} element(s)...')
+            self._thread_runner(self.viewer.load_multiple, path_list, loader, *args, **kwargs)
+            self.last_dir = QFileInfo(path_list[-1]).absoluteDir().absolutePath()
 
-    def _dialog_load_folder(self, method: classmethod, hint: str, *args, **kwargs) -> None:
+    def _dialog_load_folder(self, loader: classmethod, *args, **kwargs) -> None:
         path = QFileDialog.getExistingDirectory(
             parent=self,
             directory=self.last_dir,
             options=QFileDialog.ShowDirsOnly)
 
         # Execute method
-        self._threaded_load(method, path, *args, **kwargs)
-        self.last_dir = path
+        if bool(path):
+            self.statusBar.showMessage('Loading folder...')
+            self._thread_runner(loader, path, *args, **kwargs)
+            self.last_dir = path
 
     def _dialog_export_element(self, _id: int, filters: str, method: classmethod) -> None:
+        proposed_path = QDir(self.last_dir).filePath(self.viewer.get_drawable(_id).element.name)
+
         (path, selected_filter) = QFileDialog.getSaveFileName(
             parent=self,
-            directory=self.viewer.get_drawable(_id).element.name,
+            directory=proposed_path,
             filter=filters)
 
-        if path != '':
-            # Execute method
-            self._threaded_export(method, path, _id)
+        # Execute method
+        if bool(path):
+            self.statusBar.showMessage('Exporting...')
+            self._thread_runner(method, path, _id)
 
     """
-    Slots for pop-up dialogs
+    Slots for progress updates
     """
-    def handle_camera(self) -> None:
-        dialog = CameraDialog(self.viewer)
+    def slot_process_updated(self, value: int) -> None:
+        self.progress_bar.setValue(value)
 
-        def update_viewer() -> None:
-            self.viewer.camera_position = dialog.camera_position
-            self.viewer.rotation_angle = dialog.rotation_angle
-            self.viewer.rotation_center = dialog.rotation_center
-            self.viewer.update()
+    def slot_process_started(self) -> None:
+        self.progress_bar.setValue(0)
+        self.progress_bar.show()
 
-        dialog.accepted.connect(update_viewer)
-        dialog.show()
+    def slot_process_finished(self,) -> None:
+        self.progress_bar.hide()
 
-    def handle_screenshot(self) -> None:
+    """
+    Slot for screenshots
+    """
+    def slot_screenshot(self) -> None:
         (path, selected_filter) = QFileDialog.getSaveFileName(
             parent=self.viewer,
             directory=f'BlastSight Screenshot ({datetime.now().strftime("%Y%m%d-%H%M%S")})',
@@ -334,34 +368,34 @@ class MainWindow(QMainWindow):
     Slots for loading files
     """
     def dialog_load_mesh(self) -> None:
-        self._dialog_load_element(method=self.viewer.mesh_by_path, hint='mesh')
+        self._dialog_load_element(loader=self.viewer.load_mesh, hint='mesh')
 
     def dialog_load_blocks(self) -> None:
-        self._dialog_load_element(method=self.viewer.blocks_by_path, hint='block')
+        self._dialog_load_element(loader=self.viewer.load_blocks, hint='block')
 
     def dialog_load_points(self) -> None:
-        self._dialog_load_element(method=self.viewer.points_by_path, hint='point')
+        self._dialog_load_element(loader=self.viewer.load_points, hint='point')
 
     def dialog_load_lines(self) -> None:
-        self._dialog_load_element(method=self.viewer.lines_by_path, hint='line')
+        self._dialog_load_element(loader=self.viewer.load_lines, hint='line')
 
     def dialog_load_tubes(self) -> None:
-        self._dialog_load_element(method=self.viewer.tubes_by_path, hint='tube')
+        self._dialog_load_element(loader=self.viewer.load_tubes, hint='tube')
 
     def dialog_load_mesh_folder(self) -> None:
-        self._dialog_load_folder(method=self.viewer.meshes_by_folder_path, hint='mesh')
+        self._dialog_load_folder(loader=self.viewer.load_mesh_folder)
 
     def dialog_load_blocks_folder(self) -> None:
-        self._dialog_load_folder(method=self.viewer.blocks_by_folder_path, hint='block')
+        self._dialog_load_folder(loader=self.viewer.load_blocks_folder)
 
     def dialog_load_points_folder(self) -> None:
-        self._dialog_load_folder(method=self.viewer.points_by_folder_path, hint='point')
+        self._dialog_load_folder(loader=self.viewer.load_points_folder)
 
     def dialog_load_lines_folder(self) -> None:
-        self._dialog_load_folder(method=self.viewer.lines_by_folder_path, hint='line')
+        self._dialog_load_folder(loader=self.viewer.load_lines_folder)
 
     def dialog_load_tubes_folder(self) -> None:
-        self._dialog_load_folder(method=self.viewer.tubes_by_folder_path, hint='tube')
+        self._dialog_load_folder(loader=self.viewer.load_tubes_folder)
 
     """
     Slots for exporting files
