@@ -6,32 +6,35 @@
 #  See LICENSE for more info.
 
 import numpy as np
-import traceback
+import warnings
 
 from OpenGL.GL import *
 
-from qtpy.QtCore import QPoint
-from qtpy.QtCore import Signal
 from qtpy.QtCore import QFileInfo
+from qtpy.QtCore import QPoint
+from qtpy.QtCore import QTimer
+from qtpy.QtCore import Signal
 from qtpy.QtGui import QMatrix4x4
 from qtpy.QtGui import QPixmap
+from qtpy.QtGui import QQuaternion
 from qtpy.QtGui import QRegion
+from qtpy.QtGui import QVector3D
 from qtpy.QtGui import QVector4D
 from qtpy.QtWidgets import QOpenGLWidget
+from scipy.spatial.transform import Rotation
 
 from .collections.glaxiscollection import GLAxisCollection
 from .collections.glbackgroundcollection import GLBackgroundCollection
 from .collections.gldrawablecollection import GLDrawableCollection
 
 from .drawables.gldrawable import GLDrawable
-from .drawables.axisgl import AxisGL
-from .drawables.backgroundgl import BackgroundGL
 from .drawables.blockgl import BlockGL
 from .drawables.linegl import LineGL
 from .drawables.meshgl import MeshGL
 from .drawables.pointgl import PointGL
 from .drawables.tubegl import TubeGL
 
+from .drawablefactory import DrawableFactory
 from .fpscounter import FPSCounter
 
 from ..controller.detectionmode import DetectionMode
@@ -39,7 +42,6 @@ from ..controller.normalmode import NormalMode
 from ..controller.slicemode import SliceMode
 from ..controller.measurementmode import MeasurementMode
 
-from ..model import utils
 from ..model.model import Model
 
 
@@ -53,12 +55,19 @@ class IntegrableViewer(QOpenGLWidget):
 
     signal_mesh_clicked = Signal(object)
     signal_mesh_distances = Signal(object)
-    signal_mesh_sliced = Signal(object)
-    signal_blocks_sliced = Signal(object)
+    signal_slice_description = Signal(object)
+
+    signal_process_updated = Signal(int)
+    signal_process_started = Signal()
+    signal_process_finished = Signal()
 
     signal_mode_updated = Signal(str)
     signal_fps_updated = Signal(float)
 
+    signal_camera_rotated = Signal(object)
+    signal_camera_translated = Signal(object)
+    signal_center_translated = Signal(object)
+
     def __init__(self, parent=None):
         super().__init__(parent)
         self.setAcceptDrops(True)
@@ -66,35 +75,18 @@ class IntegrableViewer(QOpenGLWidget):
         # Model
         self.model = Model()
 
+        # Factory
+        self.factory = DrawableFactory(self.model)
+
         # Drawable elements
         self.axis_collection = GLAxisCollection(self)
         self.background_collection = GLBackgroundCollection(self)
         self.drawable_collection = GLDrawableCollection(self)
 
-        self.axis = AxisGL(id='AXIS')
-        self.axis.add_observer(self)
-        self.axis_collection.add(self.axis)
-
-        self.bg = BackgroundGL(id='BG')
-        self.bg.add_observer(self)
-        self.background_collection.add(self.bg)
-
-        # Signals for viewer (self)
-        self.signal_mode_updated.connect(lambda m: print(f'MODE: {m}'))
-        self.signal_file_modified.connect(self.recreate)
-        self.signal_load_success.connect(self.update_turbo)
-        self.signal_load_success.connect(self.update_autofit)
-
         # Controllers
         self.current_mode = None
         self.controllers = {}
 
-        self.add_controller(NormalMode, 'normal')
-        self.add_controller(DetectionMode, 'detection')
-        self.add_controller(SliceMode, 'slice')
-        self.add_controller(MeasurementMode, 'measurement')
-        self.set_normal_mode()
-
         # Camera/World/Projection
         self.camera = QMatrix4x4()
         self.world = QMatrix4x4()
@@ -109,13 +101,39 @@ class IntegrableViewer(QOpenGLWidget):
         self.camera_position = [0.0, 0.0, 200.0]
 
         # Extra information
-        self._turbo_rendering = False
-        self._autofit_to_screen = False
+        self._turbo = False
+        self._autofit = False
+        self._animated = False
 
         self.fov = 45.0
         self.smoothness = 2.0  # Bigger => smoother (but slower) rotations
         self.projection_mode = 'perspective'  # 'perspective'/'orthographic'
 
+        # Initialize viewer
+        self.initialize()
+
+    def initialize(self) -> None:
+        # Axis/Background
+        self.register_drawable(self.factory.axis(), self.axis_collection)
+        self.register_drawable(self.factory.background(), self.background_collection)
+
+        # Signals
+        self.signal_file_modified.connect(self.recreate)
+        self.signal_file_modified.connect(self.update_autofit)
+        self.signal_camera_rotated.connect(self.update)
+        self.signal_camera_translated.connect(self.update)
+        self.signal_center_translated.connect(self.update)
+
+        # Controllers
+        self.add_controller(NormalMode, 'normal')
+        self.add_controller(DetectionMode, 'detection')
+        self.add_controller(SliceMode, 'slice')
+        self.add_controller(MeasurementMode, 'measurement')
+        self.set_normal_mode()
+
+        # FPSCounter
+        self.fps_counter.add_callback(self.signal_fps_updated.emit)
+
     """
     Properties
     """
@@ -127,10 +145,21 @@ class IntegrableViewer(QOpenGLWidget):
     def last_drawable(self) -> GLDrawable:
         return self.get_drawable(self.last_id)
 
+    @property
+    def axis(self) -> GLDrawable:
+        return self.axis_collection.get_last()
+
+    @property
+    def background(self) -> GLDrawable:
+        return self.background_collection.get_last()
+
     @property
     def off_center(self) -> np.ndarray:
         return self.camera_position - self.rotation_center
 
+    """
+    Camera movements/rotations
+    """
     @property
     def camera_position(self) -> np.ndarray:
         return np.array([self.xCameraPos, self.yCameraPos, self.zCameraPos])
@@ -146,44 +175,112 @@ class IntegrableViewer(QOpenGLWidget):
     @camera_position.setter
     def camera_position(self, pos: list) -> None:
         self.xCameraPos, self.yCameraPos, self.zCameraPos = pos
+        self.signal_camera_translated.emit(pos)
 
     @rotation_angle.setter
     def rotation_angle(self, rot: list) -> None:
         self.xCenterRot, self.yCenterRot, self.zCenterRot = rot
+        self.signal_camera_rotated.emit(rot)
 
     @rotation_center.setter
     def rotation_center(self, center: list) -> None:
         self.xCenterPos, self.yCenterPos, self.zCenterPos = center
+        self.signal_center_translated.emit(center)
 
     """
-    Turbo/Autofit
+    API for partial camera movements/rotations
+    """
+    def translate(self, x: float, y: float, z: float) -> None:
+        self.camera_position += np.array([x, y, z])
+
+    def translate_center(self, x: float, y: float, z: float) -> None:
+        self.rotation_center += np.array([x, y, z])
+
+    def rotate(self, alpha: float, beta: float, gamma: float) -> None:
+        self.rotation_angle += np.array([alpha, beta, gamma])
+        self.rotation_angle %= 360
+
+    """
+    API for the camera (animated)
+    """
+    def get_camera_position(self) -> np.ndarray:
+        return self.camera_position
+
+    def get_rotation_center(self) -> np.ndarray:
+        return self.rotation_center
+
+    def get_rotation_angle(self) -> np.ndarray:
+        return self.rotation_angle
+
+    def set_camera_position(self, target) -> None:
+        origin = self.get_camera_position()
+
+        def setter(value):
+            self.camera_position = value
+        self.animate(origin, target, method=setter) if self.get_animated_status() else setter(target)
+
+    def set_rotation_center(self, target) -> None:
+        origin = self.get_rotation_center()
+
+        def setter(value):
+            self.rotation_center = value
+        self.animate(origin, target, method=setter) if self.get_animated_status() else setter(target)
+
+    def set_rotation_angle(self, target) -> None:
+        # Fix any difference > 180° so the camera doesn't over-rotate
+        origin = self.get_rotation_angle() % 360
+        origin[origin - target > 180] -= 360
+        origin[origin - target < -180] += 360
+
+        def setter(value):
+            self.rotation_angle = value
+        self.animate(origin, target, method=setter) if self.get_animated_status() else setter(target)
+
+    """
+    Turbo/Auto-fit/Animation
     """
     def get_turbo_status(self) -> bool:
-        return self._turbo_rendering
+        return self._turbo
 
     def get_autofit_status(self) -> bool:
-        return self._autofit_to_screen
+        return self._autofit
+
+    def get_animated_status(self) -> bool:
+        return self._animated
 
     def set_turbo_status(self, status: bool) -> None:
-        if status:
-            print('WARNING: Turbo-rendering might leave you without memory!')
-
-        self._turbo_rendering = status
+        self._turbo = status
         self.update_turbo()
 
     def set_autofit_status(self, status: bool) -> None:
-        self._autofit_to_screen = status
+        self._autofit = status
         self.update_autofit()
 
+    def set_animated_status(self, status: bool) -> None:
+        self._animated = status
+
     def update_turbo(self) -> None:
-        if self.get_turbo_status():
-            for d in self.get_all_drawables():
-                d.is_boostable = self.get_turbo_status()
+        for d in self.get_all_drawables():
+            d.is_boostable = self.get_turbo_status()
 
     def update_autofit(self) -> None:
         if self.get_autofit_status():
             self.fit_to_screen()
 
+    def animate(self, start, end, method: callable, frames: int = 20) -> callable:
+        timer = QTimer(self)
+        linspace = iter(np.linspace(start, end, frames))
+
+        def animation_per_frame():
+            try:
+                method(next(linspace))
+                self.update()
+            except StopIteration:
+                timer.stop()
+
+        timer.timeout.connect(animation_per_frame)
+        timer.start(1.0 / 60.0)
+
     """
     Projections
     """
@@ -196,113 +293,159 @@ class IntegrableViewer(QOpenGLWidget):
         self.update()
 
     """
-    Generator methods
+    Registration methods
     """
-    def generate_drawable(self, generator: callable) -> GLDrawable or None:
-        try:
-            drawable = generator()
-            self.signal_load_success.emit(drawable.id)
-
-            return drawable
-        except Exception:
+    def register_drawable(self, drawable, collection=None):
+        if drawable is None:
             self.signal_load_failure.emit()
-            traceback.print_exc()
-
             return None
 
-    def generate_mesh(self, generator: callable, *args, **kwargs) -> MeshGL:
-        return self.generate_drawable(lambda: MeshGL(generator(*args, **kwargs), *args, **kwargs))
+        if collection is None:
+            collection = self.drawable_collection
 
-    def generate_blocks(self, generator: callable, *args, **kwargs) -> BlockGL:
-        return self.generate_drawable(lambda: BlockGL(generator(*args, **kwargs), *args, **kwargs))
+        # Register in collection
+        drawable.add_observer(self)
+        collection.add(drawable)
 
-    def generate_points(self, generator: callable, *args, **kwargs) -> PointGL:
-        return self.generate_drawable(lambda: PointGL(generator(*args, **kwargs), *args, **kwargs))
-
-    def generate_lines(self, generator: callable, *args, **kwargs) -> LineGL:
-        return self.generate_drawable(lambda: LineGL(generator(*args, **kwargs), *args, **kwargs))
-
-    def generate_tubes(self, generator: callable, *args, **kwargs) -> TubeGL:
-        return self.generate_drawable(lambda: TubeGL(generator(*args, **kwargs), *args, **kwargs))
-
-    """
-    Register methods
-    """
-    def register_drawable(self, drawable):
-        if drawable is not None:
-            drawable.add_observer(self)
-            self.drawable_collection.add(drawable)
-            self.signal_file_modified.emit()
+        # Emit success signals
+        self.signal_file_modified.emit()
+        self.signal_load_success.emit(drawable.id)
 
         return drawable
 
-    def register_folder(self, path: str, generator: callable, *args, **kwargs) -> list:
-        path_list = self.model.get_paths_from_directory(path)
+    """
+    Load methods by arguments
+    """
+    def mesh(self, *args, **kwargs) -> MeshGL:
+        return self.register_drawable(self.factory.mesh(*args, **kwargs))
 
-        # We'll block viewer.register_drawable's signal_file_modified until last item has been loaded.
-        self.blockSignals(True)
-        loaded = [d for d in [generator(path, *args, **kwargs) for path in path_list] if d is not None]
-        self.blockSignals(False)
+    def blocks(self, *args, **kwargs) -> BlockGL:
+        return self.register_drawable(self.factory.blocks(*args, **kwargs))
 
-        # We'll manually emit the signals we didn't emit before.
+    def points(self, *args, **kwargs) -> PointGL:
+        return self.register_drawable(self.factory.points(*args, **kwargs))
+
+    def lines(self, *args, **kwargs) -> LineGL:
+        return self.register_drawable(self.factory.lines(*args, **kwargs))
+
+    def tubes(self, *args, **kwargs) -> TubeGL:
+        return self.register_drawable(self.factory.tubes(*args, **kwargs))
+
+    """
+    Load methods by path
+    """
+    def load_mesh(self, path: str, *args, **kwargs) -> MeshGL:
+        return self.register_drawable(self.factory.load_mesh(path, *args, **kwargs))
+
+    def load_blocks(self, path: str, *args, **kwargs) -> BlockGL:
+        return self.register_drawable(self.factory.load_blocks(path, *args, **kwargs))
+
+    def load_points(self, path: str, *args, **kwargs) -> PointGL:
+        return self.register_drawable(self.factory.load_points(path, *args, **kwargs))
+
+    def load_lines(self, path: str, *args, **kwargs) -> LineGL:
+        return self.register_drawable(self.factory.load_lines(path, *args, **kwargs))
+
+    def load_tubes(self, path: str, *args, **kwargs) -> TubeGL:
+        return self.register_drawable(self.factory.load_tubes(path, *args, **kwargs))
+
+    def load_mesh_folder(self, path: str, *args, **kwargs) -> list:
+        return self.load_folder(path, self.load_mesh, *args, **kwargs)
+
+    def load_blocks_folder(self, path: str, *args, **kwargs) -> list:
+        return self.load_folder(path, self.load_blocks, *args, **kwargs)
+
+    def load_points_folder(self, path: str, *args, **kwargs) -> list:
+        return self.load_folder(path, self.load_points, *args, **kwargs)
+
+    def load_lines_folder(self, path: str, *args, **kwargs) -> list:
+        return self.load_folder(path, self.load_lines, *args, **kwargs)
+
+    def load_tubes_folder(self, path: str, *args, **kwargs) -> list:
+        return self.load_folder(path, self.load_tubes, *args, **kwargs)
+
+    """
+    Generalized loaders
+    """
+    def load_folder(self, path: str, loader: callable, *args, **kwargs) -> list:
+        return self.load_multiple(self.model.get_paths_from_directory(path), loader, *args, **kwargs)
+
+    def load_multiple(self, path_list: list, loader: callable, *args, **kwargs) -> list:
+        # Do not process if loading an empty path list
+        if len(path_list) == 0:
+            return []
+
+        self.signal_process_started.emit()
+        loaded = []
+
+        for i, path in enumerate(path_list, 1):
+            # Block only loaded signals
+            self.blockSignals(True)
+            drawable = loader(path, *args, **kwargs)
+            self.blockSignals(False)
+
+            if drawable is not None:
+                loaded.append(drawable)
+
+            self.signal_process_updated.emit(int(100 * i / len(path_list)))
+
+        # Emit signals depending on results
         if len(loaded) > 0:
             self.signal_file_modified.emit()
             self.signal_load_success.emit(self.last_id)
         else:
             self.signal_load_failure.emit()
 
+        self.signal_process_finished.emit()
+
         return loaded
 
     """
-    Load methods by arguments
+    Load methods by path (DEPRECATED)
     """
-    def mesh(self, *args, **kwargs) -> MeshGL:
-        return self.register_drawable(self.generate_mesh(self.model.mesh, *args, **kwargs))
+    @staticmethod
+    def _deprecated_message(log: callable, old: str, new: str) -> None:
+        log(f'{old} is deprecated. Use {new} instead.', DeprecationWarning, 2)
 
-    def blocks(self, *args, **kwargs) -> BlockGL:
-        return self.register_drawable(self.generate_blocks(self.model.blocks, *args, **kwargs))
-
-    def points(self, *args, **kwargs) -> PointGL:
-        return self.register_drawable(self.generate_points(self.model.points, *args, **kwargs))
-
-    def lines(self, *args, **kwargs) -> LineGL:
-        return self.register_drawable(self.generate_lines(self.model.lines, *args, **kwargs))
-
-    def tubes(self, *args, **kwargs) -> TubeGL:
-        return self.register_drawable(self.generate_tubes(self.model.tubes, *args, **kwargs))
-
-    """
-    Load methods by path
-    """
     def mesh_by_path(self, path: str, *args, **kwargs) -> MeshGL:
-        return self.register_drawable(self.generate_mesh(self.model.mesh_by_path, path, *args, **kwargs))
+        self._deprecated_message(warnings.warn, 'mesh_by_path()', 'load_mesh()')
+        return self.load_mesh(path, *args, **kwargs)
 
     def blocks_by_path(self, path: str, *args, **kwargs) -> BlockGL:
-        return self.register_drawable(self.generate_blocks(self.model.blocks_by_path, path, *args, **kwargs))
+        self._deprecated_message(warnings.warn, 'blocks_by_path()', 'load_blocks()')
+        return self.load_blocks(path, *args, **kwargs)
 
     def points_by_path(self, path: str, *args, **kwargs) -> PointGL:
-        return self.register_drawable(self.generate_points(self.model.points_by_path, path, *args, **kwargs))
+        self._deprecated_message(warnings.warn, 'points_by_path()', 'load_points()')
+        return self.load_points(path, *args, **kwargs)
 
     def lines_by_path(self, path: str, *args, **kwargs) -> LineGL:
-        return self.register_drawable(self.generate_lines(self.model.lines_by_path, path, *args, **kwargs))
+        self._deprecated_message(warnings.warn, 'lines_by_path()', 'load_lines()')
+        return self.load_lines(path, *args, **kwargs)
 
     def tubes_by_path(self, path: str, *args, **kwargs) -> TubeGL:
-        return self.register_drawable(self.generate_tubes(self.model.tubes_by_path, path, *args, **kwargs))
+        self._deprecated_message(warnings.warn, 'tubes_by_path()', 'load_tubes()')
+        return self.load_tubes(path, *args, **kwargs)
 
     def meshes_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        return self.register_folder(path, self.mesh_by_path, *args, **kwargs)
+        self._deprecated_message(warnings.warn, 'meshes_by_folder_path()', 'load_mesh_folder()')
+        return self.load_mesh_folder(path, *args, **kwargs)
 
     def blocks_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        return self.register_folder(path, self.blocks_by_path, *args, **kwargs)
+        self._deprecated_folder(warnings.warn, 'blocks_by_folder_path()', 'load_blocks_folder()')
+        return self.load_blocks_folder(path, *args, **kwargs)
 
     def points_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        return self.register_folder(path, self.points_by_path, *args, **kwargs)
+        self._deprecated_folder(warnings.warn, 'points_by_folder_path()', 'load_points_folder()')
+        return self.load_points_folder(path, *args, **kwargs)
 
     def lines_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        return self.register_folder(path, self.lines_by_path, *args, **kwargs)
+        self._deprecated_folder(warnings.warn, 'lines_by_folder_path()', 'load_lines_folder()')
+        return self.load_lines_folder(path, *args, **kwargs)
 
     def tubes_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        return self.register_folder(path, self.tubes_by_path, *args, **kwargs)
+        self._deprecated_folder(warnings.warn, 'tubes_by_folder_path()', 'load_tubes_folder()')
+        return self.load_tubes_folder(path, *args, **kwargs)
 
     """
     Export methods
@@ -380,39 +523,37 @@ class IntegrableViewer(QOpenGLWidget):
     """
     def camera_at(self, _id: int) -> None:
         self.fit_to_bounds(*self.get_drawable(_id).element.bounding_box)
-        self.update()
 
     def plan_view(self) -> None:
-        self.rotation_angle = [0.0, 0.0, 0.0]
-        self.update()
+        self.set_rotation_angle([0.0, 0.0, 0.0])
 
     def north_view(self) -> None:
-        self.rotation_angle = [270.0, 0.0, 270.0]
-        self.update()
+        self.set_rotation_angle([270.0, 0.0, 270.0])
 
     def east_view(self) -> None:
-        self.rotation_angle = [270.0, 0.0, 0.0]
-        self.update()
+        self.set_rotation_angle([270.0, 0.0, 0.0])
 
     def fit_to_bounds(self, min_bound: np.ndarray, max_bound: np.ndarray) -> None:
         center = (min_bound + max_bound) / 2
         self.rotation_center = center
-        self.camera_position = center
 
         # Put the camera in a position that allow us to see between the boundaries.
-        # In perspective projection, we need a clever trigonometric calculation.
         md = np.max(np.diff([min_bound, max_bound], axis=0))
         aspect = self.width() / self.height()
         fov_rad = self.fov * np.pi / 180.0
-        dist = (md / np.tan(fov_rad / 2) + md) / 2.0
-        camera_shift = dist * max(1.0, 1.0 / aspect)
+
+        # In perspective projection, we need a clever trigonometric calculation.
+        if self.projection_mode == 'perspective':
+            dist = (md / np.tan(fov_rad / 2) + md) / 2.0
+            z_shift = dist * max(1.0, 1.0 / aspect)
 
         # But in orthographic projection, it's more direct.
-        if self.projection_mode == 'orthographic':
+        else:
             dist = md / 2
-            camera_shift = dist * max(1.0, aspect)
+            z_shift = dist * max(1.0, aspect)
 
-        self.zCameraPos += 1.1 * camera_shift
+        camera_shift = center + np.array([0.0, 0.0, 1.1 * z_shift])
+        self.set_camera_position(camera_shift)
         self.update()
 
     def fit_to_screen(self) -> None:
@@ -430,7 +571,6 @@ class IntegrableViewer(QOpenGLWidget):
             max_all = np.max((max_all, max_bound), axis=0)
 
         self.fit_to_bounds(min_all, max_all)
-        self.update()
 
     """
     Internal methods
@@ -475,14 +615,15 @@ class IntegrableViewer(QOpenGLWidget):
         self.axis_collection.draw(self.proj, self.camera, self.world)
 
         # Tick FPS counter
-        self.fps_counter.tick(callback=self.signal_fps_updated.emit)
+        self.fps_counter.tick()
 
     def resizeGL(self, w: float, h: float) -> None:
+        aspect = w / h
+
         if self.projection_mode == 'perspective':
-            self.proj.perspective(self.fov, (w / h), 1.0, 100000.0)
+            self.proj.perspective(self.fov, aspect, 1.0, 100000.0)
         elif self.projection_mode == 'orthographic':
             z = self.off_center[2]
-            aspect = w / h
             self.proj.ortho(-z, z, -z / aspect, z / aspect, 0.0, 100000.0)
 
     """
@@ -527,9 +668,8 @@ class IntegrableViewer(QOpenGLWidget):
         ray = ray_world.normalized()
         return np.array([ray.x(), ray.y(), ray.z()])
 
-    @staticmethod
-    def get_origin(model: QMatrix4x4, view: QMatrix4x4) -> np.ndarray:
-        origin = (view * model).inverted()[0].column(3).toVector3D()
+    def get_origin(self) -> np.ndarray:
+        origin = (self.camera * self.world).inverted()[0].column(3).toVector3D()
         return np.array([origin.x(), origin.y(), origin.z()])
 
     def ray_from_click(self, x: float, y: float, z: float) -> np.ndarray:
@@ -544,7 +684,7 @@ class IntegrableViewer(QOpenGLWidget):
     def origin_from_click(self, x: float, y: float, z: float) -> np.ndarray:
         # Perspective projection is straightforward
         if self.projection_mode == 'perspective':
-            return self.get_origin(self.world, self.camera)
+            return self.get_origin()
 
         # Orthographic projection needs a bit more of vector arithmetic.
         # A click in the center of the screen gives us the perfect ray,
@@ -563,49 +703,63 @@ class IntegrableViewer(QOpenGLWidget):
 
         # Hack to get the origin when the click is not exactly at center of screen.
         self.camera.translate(*-offset)
-        origin = self.get_origin(self.world, self.camera)
+        origin = self.get_origin()
         self.camera.translate(*offset)
 
         return origin
 
-    def slice_meshes(self, origin: np.ndarray, plane_normal: np.ndarray) -> None:
-        # Slicing all *visible* meshes
-        meshes = [m.element for m in self.drawable_collection.filter(MeshGL) if m.is_visible]
-
-        results = self.model.slice_meshes(origin, plane_normal, meshes)
-        self.signal_mesh_sliced.emit(results)
-
-    def slice_blocks(self, origin: np.ndarray, plane_normal: np.ndarray) -> None:
-        # Slicing all blocks
-        blocks = [m.element for m in self.drawable_collection.filter(BlockGL)]
-
-        results = self.model.slice_blocks(origin, plane_normal, blocks)
-        self.signal_blocks_sliced.emit(results)
-
-    def slice_drawables(self, origin: np.ndarray, plane_normal: np.ndarray) -> None:
-        self.slice_meshes(origin, plane_normal)
-        self.slice_blocks(origin, plane_normal)
-
-    def slice_from_rays(self, origin_list: list, ray_list: list) -> None:
-        # A plane is created from `origin` and `ray_list`.
-        # In perspective projection, the origin is the same.
-        origin = origin_list[0]
-
+    def get_normal(self, origin_list, ray_list: list) -> np.ndarray:
         if self.projection_mode == 'perspective':
             # Perspective: Same origins, different rays
-            plane_normal = np.cross(*ray_list)
+            normal = np.cross(*ray_list)
         else:
             # Orthographic: Same rays, different origins
             origin_diff = np.diff(origin_list, axis=0)[0]
-            plane_normal = np.cross(ray_list[0], origin_diff)
+            normal = np.cross(ray_list[0], origin_diff)
 
-        plane_normal /= np.linalg.norm(plane_normal)
+        return normal / np.linalg.norm(normal)
 
-        # Slice drawables
-        self.slice_drawables(origin, plane_normal)
+    @staticmethod
+    def angles_from_vectors(normal: np.ndarray, up: np.ndarray) -> np.ndarray:
+        # Returns a list of angles that allows the normal to look directly at the camera
+        # Extrinsic: xyz / Intrinsic: XYZ (Tait-Bryan angles)
+        # In self.paintGL, we're currently using the XYZ intrinsic rotation order.
 
-        # Auto-exit the slice mode for now
-        self.set_normal_mode()
+        rotmat = QQuaternion.fromDirection(QVector3D(*normal), QVector3D(*up)).toRotationMatrix().data()
+        return Rotation.from_matrix(np.array(rotmat).reshape((3, 3))).as_euler('XYZ', degrees=True)
+
+    def set_camera_from_vectors(self, normal: np.ndarray, up: np.ndarray) -> None:
+        # Auto-moves the camera using the normal as direction
+        self.set_rotation_angle(self.angles_from_vectors(normal, up))
+
+    def cross_section(self, origin_list: list, ray_list: list) -> None:
+        # A plane is created from `origin` and `ray_list`.
+        # In perspective projection, the origin is the same.
+        origin = origin_list[0]
+        normal = self.get_normal(origin_list, ray_list)
+        up = np.cross(ray_list[0], normal)
+        up /= np.linalg.norm(up)
+
+        # Emit description of the slice
+        self.signal_slice_description.emit({
+            'origin': origin,
+            'normal': normal,
+            'up': up,
+        })
+
+    def slice_meshes(self, origin: np.ndarray, normal: np.ndarray, include_hidden: bool = False) -> list:
+        # By default, slice only visible meshes
+        meshes = [m.element for m in self.drawable_collection.filter(MeshGL)
+                  if m.is_visible or include_hidden]
+
+        return self.model.slice_meshes(origin, normal, meshes)
+
+    def slice_blocks(self, origin: np.ndarray, normal: np.ndarray, include_hidden: bool = True) -> list:
+        # By default, slice visible and hidden blocks
+        blocks = [m.element for m in self.drawable_collection.filter(BlockGL)
+                  if m.is_visible or include_hidden]
+
+        return self.model.slice_blocks(origin, normal, blocks)
 
     def measure_from_rays(self, origin_list: list, ray_list: list) -> None:
         meshes = [m.element for m in self.drawable_collection.filter(MeshGL) if m.is_visible]
@@ -674,8 +828,7 @@ class IntegrableViewer(QOpenGLWidget):
     def dropEvent(self, event, *args, **kwargs) -> None:
         for url in event.mimeData().urls():
             path = url.toLocalFile()
-
             if QFileInfo(path).isDir():
-                self.meshes_by_folder_path(path)
+                self.load_folder(path, self.load_mesh)
             else:
-                self.mesh_by_path(path)
+                self.load_mesh(path)
